@@ -5,12 +5,16 @@
 #include "gt-twitch.h"
 #include "gt-player.h"
 #include "gt-player-clutter.h"
+//#include "gt-player-mpv.h"
 #include "gt-player-header-bar.h"
 #include "gt-browse-header-bar.h"
 #include "gt-channels-view.h"
 #include "gt-games-view.h"
 #include "gt-favourites-view.h"
 #include "gt-settings-dlg.h"
+#include "gt-twitch-login-dlg.h"
+#include "gt-twitch-channel-info-dlg.h"
+#include "gt-chat.h"
 #include "gt-enums.h"
 #include "utils.h"
 #include "config.h"
@@ -24,31 +28,33 @@ typedef struct
     GtkWidget* channels_view;
     GtkWidget* games_view;
     GtkWidget* favourites_view;
-    GtkWidget* player;
     GtkWidget* header_stack;
     GtkWidget* browse_stack;
     GtkWidget* player_header_bar;
     GtkWidget* browse_header_bar;
     GtkWidget* browse_stack_switcher;
+    GtkWidget* chat_view;
 
     GtkWidget* info_revealer;
     GtkWidget* info_label;
     GtkWidget* info_bar;
+    GtkWidget* info_bar_yes_button;
+
+    GtSettingsDlg* settings_dlg;
 
     gboolean fullscreen;
+
+    GtChannel* open_channel;
 } GtWinPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE(GtWin, gt_win, GTK_TYPE_APPLICATION_WINDOW)
 
-enum 
+enum
 {
     PROP_0,
     PROP_CHANNELS_VIEW,
     PROP_GAMES_VIEW,
     PROP_FULLSCREEN,
-    PROP_SHOWING_CHANNELS, //TODO: Get rid of these "showing" properties
-    PROP_SHOWING_FAVOURITES,
-    PROP_SHOWING_GAMES_VIEW,
     PROP_VISIBLE_VIEW,
     NUM_PROPS
 };
@@ -58,32 +64,12 @@ static GParamSpec* props[NUM_PROPS];
 GtWin*
 gt_win_new(GtApp* app)
 {
-    return g_object_new(GT_TYPE_WIN, 
+    return g_object_new(GT_TYPE_WIN,
                         "application", app,
+                        "show-menubar", FALSE,
                         NULL);
 }
 
-static void
-player_set_quality_cb(GSimpleAction* action,
-                      GVariant* par,
-                      gpointer udata)
-{
-    GtWin* self = GT_WIN(udata);
-    GtWinPrivate* priv = gt_win_get_instance_private(self);
-    GEnumClass* eclass;
-
-    eclass = g_type_class_ref(GT_TYPE_TWITCH_STREAM_QUALITY);
-
-    GEnumValue* qual = g_enum_get_value_by_nick(eclass, 
-                                                g_variant_get_string(par, NULL));
-
-    gt_player_set_quality(GT_PLAYER(priv->player), qual->value);
-
-    g_simple_action_set_state(action, par);
-
-    g_type_class_unref(eclass);
-}
-
 static void
 show_about_cb(GSimpleAction* action,
               GVariant* par,
@@ -91,21 +77,31 @@ show_about_cb(GSimpleAction* action,
 {
     GtWin* self = GT_WIN(udata);
     GtWinPrivate* priv = gt_win_get_instance_private(self);
+    GtkWidget* about_dlg = NULL;
 
-    char* authors[] = {"Vincent Szolnoky", NULL};
+    const char* authors[] = {"Vincent Szolnoky", NULL};
+    const char* contributors[] = {"Dimitrios Christidis", NULL};
 
-    gtk_show_about_dialog(GTK_WINDOW(self),
-                          "version", GT_VERSION,
-                          "program-name", "GNOME Twitch",
-                          "authors", &authors,
-                          "license-type", GTK_LICENSE_GPL_3_0,
-                          "copyright", "Copyright © 2015 Vincent Szolnoky",
-                          "comments", _("Enjoy Twitch on your GNU/Linux desktop"),
-                          "logo-icon-name", "gnome-twitch",
-                          "website", "https://github.com/Ippytraxx/gnome-twitch",
-                          "website-label", "Github",
-                          "translator-credits", _("translator-credits"),
-                          NULL);
+    about_dlg = gtk_about_dialog_new();
+
+    g_object_set(about_dlg,
+                 "version", GT_VERSION,
+                 "program-name", "GNOME Twitch",
+                 "authors", &authors,
+                 "license-type", GTK_LICENSE_GPL_3_0,
+                 "copyright", "Copyright © 2015 Vincent Szolnoky",
+                 "comments", _("Enjoy Twitch on your GNU/Linux desktop"),
+                 "logo-icon-name", "gnome-twitch",
+                 "website", "https://github.com/Ippytraxx/gnome-twitch",
+                 "website-label", "GitHub",
+                 "translator-credits", _("translator-credits"),
+                 NULL);
+
+    gtk_about_dialog_add_credit_section(GTK_ABOUT_DIALOG(about_dlg), _("Contributors"), contributors);
+    gtk_window_set_transient_for(GTK_WINDOW(about_dlg), GTK_WINDOW(self));
+
+    gtk_dialog_run(GTK_DIALOG(about_dlg));
+    gtk_widget_destroy(about_dlg);
 }
 
 static void
@@ -116,11 +112,101 @@ show_settings_cb(GSimpleAction* action,
     GtWin* self = GT_WIN(udata);
     GtWinPrivate* priv = gt_win_get_instance_private(self);
 
-    GtSettingsDlg* settings_dlg = gt_settings_dlg_new(self);
+    if (!priv->settings_dlg)
+    {
+        priv->settings_dlg = gt_settings_dlg_new(self);
+        g_object_add_weak_pointer(G_OBJECT(priv->settings_dlg), (gpointer *) &priv->settings_dlg);
+    }
 
-    gtk_window_present(GTK_WINDOW(settings_dlg));
+    gtk_window_present(GTK_WINDOW(priv->settings_dlg));
 }
 
+static void
+refresh_login_cb(GtkInfoBar* info_bar,
+                 gint res,
+                 gpointer udata)
+{
+    GtWin* self = GT_WIN(udata);
+    GtWinPrivate* priv = gt_win_get_instance_private(self);
+
+    switch (res)
+    {
+        case GTK_RESPONSE_YES:
+            gtk_window_present(GTK_WINDOW(gt_twitch_login_dlg_new(GTK_WINDOW(self))));
+            break;
+    }
+
+    gtk_revealer_set_reveal_child(GTK_REVEALER(priv->info_revealer), FALSE);
+    gtk_widget_set_visible(priv->info_bar_yes_button, FALSE);
+    g_signal_handlers_disconnect_by_func(info_bar, refresh_login_cb, udata);
+}
+
+static void
+close_info_bar_cb(GtkInfoBar* bar,
+                  gint res,
+                  gpointer udata)
+{
+    GtWin* self = GT_WIN(udata);
+    GtWinPrivate* priv = gt_win_get_instance_private(self);
+
+    if (res == GTK_RESPONSE_CLOSE)
+        gtk_revealer_set_reveal_child(GTK_REVEALER(priv->info_revealer), FALSE);
+}
+
+
+static void
+show_twitch_login_cb(GSimpleAction* action,
+                     GVariant* par,
+                     gpointer udata)
+{
+    GtWin* self = GT_WIN(udata);
+    GtWinPrivate* priv = gt_win_get_instance_private(self);
+    gchar* oauth_token;
+    gchar* user_name;
+
+    g_object_get(main_app,
+                 "oauth-token", &oauth_token,
+                 "user-name", &user_name,
+                 NULL);
+
+    if (oauth_token && user_name &&
+        strlen(oauth_token) > 0 && strlen(user_name) > 0)
+    {
+        gtk_widget_set_visible(priv->info_bar_yes_button, TRUE);
+        gtk_label_set_text(GTK_LABEL(priv->info_label), _("Already logged into Twitch, refresh login?"));
+        gtk_info_bar_set_message_type(GTK_INFO_BAR(priv->info_bar), GTK_MESSAGE_QUESTION);
+        gtk_revealer_set_reveal_child(GTK_REVEALER(priv->info_revealer), TRUE);
+
+        g_signal_connect(priv->info_bar, "response", G_CALLBACK(refresh_login_cb), self);
+    }
+    else
+    {
+        GtTwitchLoginDlg* dlg = gt_twitch_login_dlg_new(GTK_WINDOW(self));
+
+        gtk_window_present(GTK_WINDOW(dlg));
+    }
+}
+
+static void
+show_channel_info_cb(GSimpleAction* action,
+                     GVariant* arg,
+                     gpointer udata)
+{
+    GtWin* self = GT_WIN(udata);
+    GtWinPrivate* priv = gt_win_get_instance_private(self);
+    GtTwitchChannelInfoDlg* dlg = gt_twitch_channel_info_dlg_new(GTK_WINDOW(self));
+    const gchar* chan;
+
+    chan = gt_channel_get_name(priv->open_channel);
+
+    g_message("{GtWin} Showing channel info for '%s'", chan);
+
+    gtk_window_present(GTK_WINDOW(dlg));
+
+    gt_twitch_channel_info_dlg_load_channel(dlg, chan);
+}
+
+
 static void
 refresh_view_cb(GSimpleAction* action,
                 GVariant* arg,
@@ -130,9 +216,9 @@ refresh_view_cb(GSimpleAction* action,
     GtWinPrivate* priv = gt_win_get_instance_private(self);
 
     if (gtk_stack_get_visible_child(GTK_STACK(priv->browse_stack)) == priv->channels_view)
-        gt_channels_view_refresh(GT_CHANNELS_VIEW(priv->channels_view)); 
+        gt_channels_view_refresh(GT_CHANNELS_VIEW(priv->channels_view));
     else if (gtk_stack_get_visible_child(GTK_STACK(priv->browse_stack)) == priv->games_view)
-        gt_games_view_refresh(GT_GAMES_VIEW(priv->games_view)); 
+        gt_games_view_refresh(GT_GAMES_VIEW(priv->games_view));
 }
 
 static void
@@ -144,9 +230,9 @@ show_view_default_cb(GSimpleAction* action,
     GtWinPrivate* priv = gt_win_get_instance_private(self);
 
     if (gtk_stack_get_visible_child(GTK_STACK(priv->browse_stack)) == priv->channels_view)
-        gt_channels_view_show_type(GT_CHANNELS_VIEW(priv->channels_view), GT_CHANNELS_CONTAINER_TYPE_TOP); 
+        gt_channels_view_show_type(GT_CHANNELS_VIEW(priv->channels_view), GT_CHANNELS_CONTAINER_TYPE_TOP);
     else if (gtk_stack_get_visible_child(GTK_STACK(priv->browse_stack)) == priv->games_view)
-        gt_games_view_show_type(GT_GAMES_VIEW(priv->games_view), GT_GAMES_CONTAINER_TYPE_TOP); 
+        gt_games_view_show_type(GT_GAMES_VIEW(priv->games_view), GT_GAMES_CONTAINER_TYPE_TOP);
 }
 
 static gboolean
@@ -156,37 +242,97 @@ key_press_cb(GtkWidget* widget,
 {
     GtWin* self = GT_WIN(udata);
     GtWinPrivate* priv = gt_win_get_instance_private(self);
+    GdkModifierType modifiers = gtk_accelerator_get_default_mod_mask();
     gboolean playing;
+    GAction *action;
 
-    g_object_get(priv->player, "playing", &playing, NULL);
+    g_object_get(self->player, "playing", &playing, NULL);
 
-    if (evt->keyval & GDK_KEY_space)
+    if (MAIN_VISIBLE_CHILD == self->player)
     {
-        if (MAIN_VISIBLE_CHILD == priv->player)
+        if (evt->keyval == GDK_KEY_Escape)
         {
-            if (playing)
-                gt_player_stop(GT_PLAYER(priv->player));
-            else
-                gt_player_play(GT_PLAYER(priv->player));
-        }
-    }
-    else if (evt->keyval & GDK_KEY_Escape)
-    {
-        if (MAIN_VISIBLE_CHILD == priv->player)
             if (priv->fullscreen)
                 gtk_window_unfullscreen(GTK_WINDOW(self));
+            else
+            {
+                action = g_action_map_lookup_action(G_ACTION_MAP(self), "close_player");
+                g_action_activate(action, NULL);
+            }
+        }
+        else if (evt->keyval == GDK_KEY_f)
+        {
+            if (priv->fullscreen)
+                gtk_window_unfullscreen(GTK_WINDOW(self));
+            else
+                gtk_window_fullscreen(GTK_WINDOW(self));
+        }
+    }
+    else
+    {
+        if (evt->keyval == GDK_KEY_Escape)
+            gt_browse_header_bar_stop_search(GT_BROWSE_HEADER_BAR(priv->browse_header_bar));
+        else if (evt->keyval == GDK_KEY_f && (evt->state & modifiers) == GDK_CONTROL_MASK)
+            gt_browse_header_bar_toggle_search(GT_BROWSE_HEADER_BAR(priv->browse_header_bar));
+        else
+        {
+            GtkWidget* view = gtk_stack_get_visible_child(GTK_STACK(priv->browse_stack));
+
+            if (view == priv->channels_view)
+                gt_channels_view_handle_event(GT_CHANNELS_VIEW(priv->channels_view), (GdkEvent*) evt);
+            else if (view == priv->games_view)
+                gt_games_view_handle_event(GT_GAMES_VIEW(priv->games_view), (GdkEvent*) evt);
+            else if (view == priv->favourites_view)
+                gt_favourites_view_handle_event(GT_FAVOURITES_VIEW(priv->favourites_view), (GdkEvent* )evt);
+        }
     }
 
     return FALSE;
 }
 
-static GActionEntry win_actions[] = 
+static gboolean
+delete_cb(GtkWidget* widget,
+          GdkEvent* evt,
+          gpointer udata)
+{
+    GtWin* self = GT_WIN(udata);
+    GtWinPrivate* priv = gt_win_get_instance_private(self);
+    int width;
+    int height;
+
+    gtk_window_get_size(GTK_WINDOW(self), &width, &height);
+
+    g_settings_set_int(main_app->settings, "window-width", width);
+    g_settings_set_int(main_app->settings, "window-height", height);
+
+    return FALSE;
+}
+
+static void
+close_player_cb(GSimpleAction* action,
+                GVariant* arg,
+                gpointer udata)
+{
+    GtWin* self = GT_WIN(udata);
+    GtWinPrivate* priv = gt_win_get_instance_private(self);
+
+    gt_player_close_channel(GT_PLAYER(self->player));
+
+    gtk_stack_set_visible_child_name(GTK_STACK(priv->header_stack),
+                                     "browse");
+    gtk_stack_set_visible_child_name(GTK_STACK(priv->main_stack),
+                                     "browse");
+}
+
+static GActionEntry win_actions[] =
 {
-    {"player_set_quality", player_set_quality_cb, "s", "'source'", NULL},
     {"refresh_view", refresh_view_cb, NULL, NULL, NULL},
     {"show_view_default", show_view_default_cb, NULL, NULL, NULL},
     {"show_about", show_about_cb, NULL, NULL, NULL},
-    {"show_settings", show_settings_cb, NULL, NULL, NULL}
+    {"show_settings", show_settings_cb, NULL, NULL, NULL},
+    {"show_twitch_login", show_twitch_login_cb, NULL, NULL, NULL},
+    {"show_channel_info", show_channel_info_cb, NULL, NULL, NULL},
+    {"close_player", close_player_cb, NULL, NULL, NULL},
 };
 
 static gboolean
@@ -209,24 +355,6 @@ window_state_cb(GtkWidget* widget,
     return FALSE;
 }
 
-static void
-show_error_message(GtWin* self, const gchar* msg)
-{
-    GtWinPrivate* priv = gt_win_get_instance_private(self);
-
-    gtk_label_set_text(GTK_LABEL(priv->info_label), msg);
-    gtk_info_bar_set_message_type(GTK_INFO_BAR(priv->info_bar), GTK_MESSAGE_WARNING);
-    gtk_revealer_set_reveal_child(GTK_REVEALER(priv->info_revealer), TRUE);
-}
-
-static void
-hide_info_bar(GtWin* self)
-{
-    GtWinPrivate* priv = gt_win_get_instance_private(self);
-
-    gtk_revealer_set_reveal_child(GTK_REVEALER(priv->info_revealer), FALSE);
-}
-
 static void
 finalize(GObject* object)
 {
@@ -256,15 +384,6 @@ get_property (GObject*    obj,
         case PROP_FULLSCREEN:
             g_value_set_boolean(val, priv->fullscreen);
             break;
-        case PROP_SHOWING_CHANNELS:
-            g_value_set_boolean(val, gtk_stack_get_visible_child(GTK_STACK(priv->browse_stack)) == priv->channels_view);
-            break;
-        case PROP_SHOWING_FAVOURITES:
-            g_value_set_boolean(val, gtk_stack_get_visible_child(GTK_STACK(priv->browse_stack)) == priv->favourites_view);
-            break;
-        case PROP_SHOWING_GAMES_VIEW:
-            g_value_set_boolean(val, gtk_stack_get_visible_child(GTK_STACK(priv->browse_stack)) == priv->games_view);
-            break;
         case PROP_VISIBLE_VIEW:
             g_value_set_object(val, gtk_stack_get_visible_child(GTK_STACK(priv->browse_stack)));
             break;
@@ -316,21 +435,6 @@ gt_win_class_init(GtWinClass* klass)
                                                   "Whether window is fullscreen",
                                                   FALSE,
                                                   G_PARAM_READABLE);
-    props[PROP_SHOWING_CHANNELS] = g_param_spec_boolean("showing-channels",
-                                                        "Showing Channels",
-                                                        "Whether showing channels",
-                                                        FALSE,
-                                                        G_PARAM_READABLE);
-    props[PROP_SHOWING_FAVOURITES] = g_param_spec_boolean("showing-favourites",
-                                                          "Showing Favourites",
-                                                          "Whether showing favourites",
-                                                          FALSE,
-                                                          G_PARAM_READABLE);
-    props[PROP_SHOWING_GAMES_VIEW] = g_param_spec_boolean("showing-games-view",
-                                                          "Showing Games View",
-                                                          "Whether showing games view",
-                                                          FALSE,
-                                                          G_PARAM_READABLE);
     props[PROP_VISIBLE_VIEW] = g_param_spec_object("visible-view",
                                                    "Visible View",
                                                    "Visible View",
@@ -341,12 +445,12 @@ gt_win_class_init(GtWinClass* klass)
                                       NUM_PROPS,
                                       props);
 
-    gtk_widget_class_set_template_from_resource(GTK_WIDGET_CLASS(klass), 
+    gtk_widget_class_set_template_from_resource(GTK_WIDGET_CLASS(klass),
                                                 "/com/gnome-twitch/ui/gt-win.ui");
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, main_stack);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, channels_view);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, games_view);
-    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, player);
+    gtk_widget_class_bind_template_child(GTK_WIDGET_CLASS(klass), GtWin, player);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, header_stack);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, player_header_bar);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, browse_header_bar);
@@ -356,8 +460,7 @@ gt_win_class_init(GtWinClass* klass)
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, info_revealer);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, info_label);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, info_bar);
-
-    gtk_widget_class_bind_template_callback(GTK_WIDGET_CLASS(klass), hide_info_bar);
+    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, info_bar_yes_button);
 }
 
 static void
@@ -367,34 +470,48 @@ gt_win_init(GtWin* self)
 
     GT_TYPE_PLAYER; // Hack to load GtPlayer into the symbols table
     GT_TYPE_PLAYER_CLUTTER;
+//    GT_TYPE_PLAYER_MPV;
     GT_TYPE_PLAYER_HEADER_BAR;
     GT_TYPE_BROWSE_HEADER_BAR;
     GT_TYPE_CHANNELS_VIEW;
     GT_TYPE_GAMES_VIEW;
     GT_TYPE_FAVOURITES_VIEW;
+    GT_TYPE_CHAT;
+
+//    self->open_channel = NULL;
+
+    gtk_window_set_default_size(GTK_WINDOW(self),
+                                g_settings_get_int(main_app->settings, "window-width"),
+                                g_settings_get_int(main_app->settings, "window-height"));
 
     gtk_widget_init_template(GTK_WIDGET(self));
 
-    g_object_set(self, "application", main_app, NULL); // Another hack because GTK is bugged and resets the app menu when using custom widgets
+    gtk_window_set_default_icon_name("gnome-twitch");
+
+//    g_object_set(self, "application", main_app, NULL); // Another hack because GTK is bugged and resets the app menu when using custom widgets
 
     g_object_bind_property(priv->browse_stack, "visible-child",
                            self, "visible-view",
                            G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
 
-    /* gtk_widget_realize(GTK_WIDGET(priv->player)); */
-
     GdkScreen* screen = gdk_screen_get_default();
     GtkCssProvider* css = gtk_css_provider_new();
     gtk_css_provider_load_from_resource(css, "/com/gnome-twitch/gnome-twitch-style.css");
     gtk_style_context_add_provider_for_screen(screen, GTK_STYLE_PROVIDER(css), GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 
     g_signal_connect(self, "window-state-event", G_CALLBACK(window_state_cb), self);
-    g_signal_connect(self, "key-press-event", G_CALLBACK(key_press_cb), self);
+    g_signal_connect_after(self, "key-press-event", G_CALLBACK(key_press_cb), self);
+    g_signal_connect(self, "delete-event", G_CALLBACK(delete_cb), self);
+    g_signal_connect(priv->info_bar, "response", G_CALLBACK(close_info_bar_cb), self);
 
     g_action_map_add_action_entries(G_ACTION_MAP(self),
                                     win_actions,
                                     G_N_ELEMENTS(win_actions),
                                     self);
+
+    GtkWindowGroup* window_group = gtk_window_group_new();
+    gtk_window_group_add_window(window_group, GTK_WINDOW(self));
+    g_object_unref(window_group);
 }
 
 //TODO: Make this action
@@ -403,34 +520,16 @@ gt_win_open_channel(GtWin* self, GtChannel* chan)
 {
     GtWinPrivate* priv = gt_win_get_instance_private(self);
 
-    gchar* status; gchar* display_name; gchar* name; gchar* token; gchar* sig;
-    g_object_get(chan, 
-                 "display-name", &display_name,
-                 "name", &name,
-                 "status", &status,
-                 NULL);
+    g_clear_object(&priv->open_channel);
+    g_object_ref(chan);
+    priv->open_channel = chan;
 
-    gt_twitch_stream_access_token(main_app->twitch, name, &token, &sig);
-    GtTwitchStreamData* stream_data = gt_twitch_stream_by_quality(main_app->twitch,
-                                                                  name,
-                                                                  GT_TWITCH_STREAM_QUALITY_SOURCE,
-                                                                  token, sig);
+    gt_player_open_channel(GT_PLAYER(self->player), chan);
 
-    if (!stream_data)
-        show_error_message(self, "Error opening stream");
-    else
-    {
-        gt_player_open_channel(GT_PLAYER(priv->player), chan);
-
-        gtk_stack_set_visible_child_name(GTK_STACK(priv->main_stack),
-                                         "player");
-        gtk_stack_set_visible_child_name(GTK_STACK(priv->header_stack),
-                                         "player");
-    }
-
-    g_free(name);
-    g_free(status);
-    g_free(display_name);
+    gtk_stack_set_visible_child_name(GTK_STACK(priv->main_stack),
+                                     "player");
+    gtk_stack_set_visible_child_name(GTK_STACK(priv->header_stack),
+                                     "player");
 }
 
 //TODO: Make these actions
@@ -490,3 +589,13 @@ gt_win_get_fullscreen(GtWin* self)
 
     return priv->fullscreen;
 }
+
+void
+gt_win_show_info_message(GtWin* self, const gchar* msg)
+{
+    GtWinPrivate* priv = gt_win_get_instance_private(self);
+
+    gtk_label_set_text(GTK_LABEL(priv->info_label), msg);
+    gtk_info_bar_set_message_type(GTK_INFO_BAR(priv->info_bar), GTK_MESSAGE_INFO);
+    gtk_revealer_set_reveal_child(GTK_REVEALER(priv->info_revealer), TRUE);
+}
